from typing import Annotated
from contextlib import asynccontextmanager
import logging

from fastapi import (
    FastAPI,
    Depends,
    BackgroundTasks,
    HTTPException,
    status,
)
from fastapi.responses import FileResponse, HTMLResponse
from sqlalchemy.sql.base import ExecutableOption
from sqlalchemy.exc import NoResultFound
from sqlmodel import select
from sqlmodel.ext.asyncio.session import AsyncSession

from knoc.settings import settings
from knoc.workflows import all_wfs
from knoc.workflows.utils import ImportReport
from knoc.db import engine, aio_sql_to_df, get_session
from knoc_splynx.models import Customer

logger = logging.getLogger("knoc-splynx.api")


app = FastAPI()


@app.get("/customer")
async def list_customers():
    df = await aio_sql_to_df(
        settings.splynx.db_table_customers,
        schema=settings.splynx.db_schema,
    )
    return df.to_dict(orient="records")


@app.get("/customer/{id}")
async def customer(id: str, session: AsyncSession = Depends(get_session)) -> Customer:
    query = select(Customer).where(Customer.aa_id_erp == id)
    result = await session.exec(query)
    try:
        return result.one()
    except NoResultFound:
        raise HTTPException(status.HTTP_404_NOT_FOUND)
